import { ItemSourcePF2e } from "@item/data";
import { Size } from "@module/data";
import { BracketedValue, RuleElementSource, RuleValue } from "@module/rules/rule-element";
import { MigrationBase } from "../base";

/** Combine AE-likes altering creature size and TokenSize RuleElements into CreatureSize RuleElements */
export class Migration655CreatureTokenSizes extends MigrationBase {
    static override version = 0.655;

    private isTokenSizeRE(rule: MaybeAELike): boolean {
        return !!rule.key?.endsWith("TokenSize");
    }

    private isActorSizeAELike(rule: MaybeAELike): boolean {
        return !!rule.key?.endsWith("ActiveEffectLike") && rule.path === "system.traits.size.value";
    }

    private isBracketedValue(value: RuleValue | BracketedValue | undefined): value is BracketedValue {
        return value instanceof Object && "brackets" in value && Array.isArray(value.brackets);
    }

    private dimensionToSize: Record<string, Size | undefined> = {
        0.5: "tiny",
        2: "lg",
        3: "huge",
        4: "grg",
    };

    override async updateItem(itemSource: ItemSourcePF2e): Promise<void> {
        itemSource.system.rules ??= [];
        const rules: MaybeAELike[] = itemSource.system.rules;
        const actorSizeAELike = rules.find(this.isActorSizeAELike);
        const tokenSizeRE = rules.find(this.isTokenSizeRE);

        if (actorSizeAELike) {
            actorSizeAELike.key = "CreatureSize";
            delete actorSizeAELike.path;
            delete actorSizeAELike.mode;
        } else if (tokenSizeRE && ["number", "string", "object"].includes(typeof tokenSizeRE.value)) {
            tokenSizeRE.key = "CreatureSize";
            if (this.isBracketedValue(tokenSizeRE.value)) {
                for (const bracket of tokenSizeRE.value.brackets) {
                    if (typeof bracket.value === "number") {
                        bracket.value = this.dimensionToSize[bracket.value] ?? "med";
                    }
                }
            } else if (typeof tokenSizeRE.value === "number") {
                tokenSizeRE.value = this.dimensionToSize[tokenSizeRE.value] ?? "med";
            }
        }

        if (tokenSizeRE && tokenSizeRE.key !== "CreatureSize") {
            itemSource.system.rules.splice(itemSource.system.rules.indexOf(tokenSizeRE), 1);
        }
    }
}

type MaybeAELike = RuleElementSource & { path?: string; mode?: string };
