import {
    ActionItemPF2e,
    AncestryPF2e,
    BackgroundPF2e,
    BookPF2e,
    ArmorPF2e,
    ClassPF2e,
    ConditionPF2e,
    ConsumablePF2e,
    ContainerPF2e,
    DeityPF2e,
    EffectPF2e,
    EquipmentPF2e,
    FeatPF2e,
    HeritagePF2e,
    KitPF2e,
    LorePF2e,
    MeleePF2e,
    SpellPF2e,
    SpellcastingEntryPF2e,
    TreasurePF2e,
    WeaponPF2e,
} from "@item";
import { CharacterPF2e, NPCPF2e, FamiliarPF2e, HazardPF2e, LootPF2e, VehiclePF2e } from "@actor";
import { ConditionSlug } from "@item/condition/data";
import { WEAPON_PROPERTY_RUNES } from "@item/runes";
import { PreciousMaterialGrade } from "@item/physical/types";
import { DamageCategory, DamageType } from "@system/damage/calculation";
import { ImmunityType, ResistanceType, WeaknessType } from "@actor/data/base";
import { RANGE_TRAITS } from "@item/data/values";
import { ActorType } from "@actor/data";
import { BaseWeaponType, MeleeWeaponGroup, WeaponGroup, WeaponPropertyRuneType } from "@item/weapon/types";
import enJSON from "../../../static/lang/en.json";
import { SenseAcuity, SenseType } from "@actor/creature/sense";
import {
    actionTraits,
    alignmentTraits,
    ancestryItemTraits,
    ancestryTraits,
    armorTraits,
    classTraits,
    consumableTraits,
    creatureTraits,
    damageTraits,
    elementalTraits,
    energyDamageTypes,
    equipmentTraits,
    featTraits,
    hazardTraits,
    magicSchools,
    magicTraditions,
    npcAttackTraits,
    otherArmorTags,
    otherEquipmentTags,
    otherWeaponTags,
    preciousMaterials,
    spellOtherTraits,
    spellTraits,
    vehicleTraits,
    weaponTraits,
} from "./traits";
import { JournalSheetPF2e } from "@module/journal-entry/sheet";
import { Size } from "@module/data";
import { FeatType } from "@item/feat/data";
import { DeityDomain } from "@item/deity/types";
import { sluggify } from "@util";
import { Alignment } from "@actor/creature/types";
import { WeaponReloadTime } from "@item/weapon/types";

export type StatusEffectIconTheme = "default" | "blackWhite";

const actorTypes: Record<ActorType, string> = {
    character: "ACTOR.TypeCharacter",
    familiar: "ACTOR.TypeFamiliar",
    hazard: "ACTOR.TypeHazard",
    loot: "ACTOR.TypeLoot",
    npc: "ACTOR.TypeNpc",
    vehicle: "ACTOR.TypeVehicle",
};

// Senses
const senses: Record<SenseType, string> = {
    darkvision: "PF2E.SensesDarkvision",
    echolocation: "PF2E.SensesEcholocation",
    greaterDarkvision: "PF2E.SensesGreaterDarkvision",
    lifesense: "PF2E.SensesLifesense",
    lowLightVision: "PF2E.SensesLowLightVision",
    motionsense: "PF2E.SensesMotionsense",
    scent: "PF2E.SensesScent",
    tremorsense: "PF2E.SensesTremorsense",
    wavesense: "PF2E.SensesWavesense",
};

// Sense acuity
const senseAcuity: Record<SenseAcuity, string> = {
    imprecise: "PF2E.Sense.Acuity.Imprecise",
    precise: "PF2E.Sense.Acuity.Precise",
    vague: "PF2E.Sense.Acuity.Vague",
};

const damageCategories: Record<DamageCategory, string> = {
    alignment: "PF2E.Alignment",
    adamantine: "PF2E.PreciousMaterialAdamantine",
    coldiron: "PF2E.PreciousMaterialColdIron",
    darkwood: "PF2E.PreciousMaterialDarkwood",
    energy: "PF2E.TraitEnergy",
    ghostTouch: "PF2E.WeaponPropertyRuneGhostTouch",
    mithral: "PF2E.PreciousMaterialMithral",
    orichalcum: "PF2E.PreciousMaterialOrichalcum",
    physical: "PF2E.TraitPhysical",
    precision: "PF2E.TraitPrecision",
    salt: "PF2E.TraitSalt",
    "salt-water": "PF2E.TraitSaltWater",
    silver: "PF2E.PreciousMaterialSilver",
    warpglass: "PF2E.PreciousMaterialWarpglass",
};

const physicalDamageTypes = {
    bleed: "PF2E.TraitBleed",
    bludgeoning: "PF2E.TraitBludgeoning",
    piercing: "PF2E.TraitPiercing",
    slashing: "PF2E.TraitSlashing",
};

const damageTypes: Record<DamageType, string> = {
    ...alignmentTraits,
    ...elementalTraits,
    ...energyDamageTypes,
    ...physicalDamageTypes,
    mental: "PF2E.TraitMental",
    poison: "PF2E.TraitPoison",
    untyped: "PF2E.TraitUntyped",
};

/** Non-detection- and attitude- related conditions added to the Token HUD */
const tokenHUDConditions = {
    blinded: "PF2E.ConditionTypeBlinded",
    broken: "PF2E.ConditionTypeBroken",
    clumsy: "PF2E.ConditionTypeClumsy",
    concealed: "PF2E.ConditionTypeConcealed",
    confused: "PF2E.ConditionTypeConfused",
    controlled: "PF2E.ConditionTypeControlled",
    dazzled: "PF2E.ConditionTypeDazzled",
    deafened: "PF2E.ConditionTypeDeafened",
    doomed: "PF2E.ConditionTypeDoomed",
    drained: "PF2E.ConditionTypeDrained",
    dying: "PF2E.ConditionTypeDying",
    encumbered: "PF2E.ConditionTypeEncumbered",
    enfeebled: "PF2E.ConditionTypeEnfeebled",
    fascinated: "PF2E.ConditionTypeFascinated",
    fatigued: "PF2E.ConditionTypeFatigued",
    "flat-footed": "PF2E.ConditionTypeFlatFooted",
    fleeing: "PF2E.ConditionTypeFleeing",
    frightened: "PF2E.ConditionTypeFrightened",
    grabbed: "PF2E.ConditionTypeGrabbed",
    immobilized: "PF2E.ConditionTypeImmobilized",
    invisible: "PF2E.ConditionTypeInvisible",
    paralyzed: "PF2E.ConditionTypeParalyzed",
    "persistent-damage": "PF2E.ConditionTypePersistent",
    petrified: "PF2E.ConditionTypePetrified",
    prone: "PF2E.ConditionTypeProne",
    quickened: "PF2E.ConditionTypeQuickened",
    restrained: "PF2E.ConditionTypeRestrained",
    sickened: "PF2E.ConditionTypeSickened",
    slowed: "PF2E.ConditionTypeSlowed",
    stunned: "PF2E.ConditionTypeStunned",
    stupefied: "PF2E.ConditionTypeStupefied",
    unconscious: "PF2E.ConditionTypeUnconscious",
    wounded: "PF2E.ConditionTypeWounded",
};

const conditionTypes: Record<ConditionSlug, string> = {
    ...tokenHUDConditions,
    friendly: "PF2E.ConditionTypeFriendly",
    helpful: "PF2E.ConditionTypeHelpful",
    hidden: "PF2E.ConditionTypeHidden",
    hostile: "PF2E.ConditionTypeHostile",
    indifferent: "PF2E.ConditionTypeIndifferent",
    observed: "PF2E.ConditionTypeObserved",
    undetected: "PF2E.ConditionTypeUndetected",
    unfriendly: "PF2E.ConditionTypeUnfriendly",
    unnoticed: "PF2E.ConditionTypeUnnoticed",
};

const immunityTypes: Record<ImmunityType, string> = {
    ...conditionTypes,
    ...damageCategories,
    ...damageTraits,
    ...damageTypes,
    ...magicSchools,
    "area-damage": "PF2E.TraitAreaDamage",
    auditory: "PF2E.TraitAuditory",
    confusion: "PF2E.TraitConfusion",
    "critical-hits": "PF2E.TraitCriticalHits",
    curse: "PF2E.TraitCurse",
    detection: "PF2E.TraitDetection",
    "death-effects": "PF2E.TraitDeathEffects",
    disease: "PF2E.TraitDisease",
    emotion: "PF2E.TraitEmotion",
    "fear-effects": "PF2E.TraitFearEffects",
    ghostTouch: "PF2E.WeaponPropertyRuneGhostTouch",
    healing: "PF2E.TraitHealing",
    inhaled: "PF2E.TraitInhaled",
    light: "PF2E.TraitLight",
    magic: "PF2E.TraitMagic",
    magical: "PF2E.TraitMagical",
    "nonlethal-attacks": "PF2E.TraitNonlethalAttacks",
    "nonmagical-attacks": "PF2E.TraitNonmagicalAttacks",
    "object-immunities": "PF2E.TraitObjectImmunities",
    olfactory: "PF2E.TraitOlfactory",
    polymorph: "PF2E.TraitPolymorph",
    possession: "PF2E.TraitPossession",
    scrying: "PF2E.TraitScrying",
    sleep: "PF2E.TraitSleep",
    spellDeflection: "PF2E.TraitSpellDeflection",
    "swarm-attacks": "PF2E.TraitSwarmAttacks",
    "swarm-mind": "PF2E.TraitSwarmMind",
    trip: "PF2E.TraitTrip",
    unarmed: "PF2E.TraitUnarmed",
    visual: "PF2E.TraitVisual",
};

const weaknessTypes: Record<WeaknessType, string> = {
    ...damageCategories,
    ...damageTraits,
    ...damageTypes,
    "area-damage": "PF2E.TraitAreaDamage",
    axe: "PF2E.TraitAxeVulnerability",
    "critical-hits": "PF2E.TraitCriticalHits",
    emotion: "PF2E.TraitEmotion",
    "nonlethal-attacks": "PF2E.TraitNonlethalAttacks",
    "persistent-damage": "PF2E.ConditionTypePersistent",
    "salt-water": "PF2E.TraitSaltWater",
    "splash-damage": "PF2E.TraitSplashDamage",
    "vampire-weaknesses": "PF2E.TraitVampireWeaknesses",
    vorpal: "PF2E.WeaponPropertyRuneVorpal",
    "vorpal-fear": "PF2E.TraitVorpalFear",
    "vulnerable-to-sunlight": "PF2E.TraitVulnerableToSunlight",
    unarmed: "PF2E.TraitUnarmed",
    weapons: "PF2E.TraitWeapons",
};

const resistanceTypes: Record<ResistanceType, string> = {
    ...damageCategories,
    ...damageTraits,
    ...damageTypes,
    all: "PF2E.TraitAll",
    "area-damage": "PF2E.TraitAreaDamage",
    "critical-hits": "PF2E.TraitCriticalHits",
    "nonlethal-attacks": "PF2E.TraitNonlethalAttacks",
    "persistent-damage": "PF2E.ConditionTypePersistent",
    "protean anatomy": "PF2E.TraitProteanAnatomy",
    unarmed: "PF2E.TraitUnarmed",
    vorpal: "PF2E.WeaponPropertyRuneVorpal",
    weapons: "PF2E.TraitWeapons",
};

const weaponCategories = {
    simple: "PF2E.WeaponTypeSimple",
    martial: "PF2E.WeaponTypeMartial",
    advanced: "PF2E.WeaponTypeAdvanced",
    unarmed: "PF2E.WeaponTypeUnarmed",
};

const baseWeaponTypes = Object.keys(enJSON.PF2E.Weapon.Base).reduce(
    (map, slug) => ({
        ...map,
        [slug]: `PF2E.Weapon.Base.${slug}`,
    }),
    {} as Record<BaseWeaponType, string>
);

/** Base weapon types that are considered equivalent for all rules purposes */
const equivalentWeapons = {
    "composite-longbow": "longbow",
    "composite-shortbow": "shortbow",
    "big-boom-gun": "hand-cannon",
    "spoon-gun": "hand-cannon",
} as const;

const rangeDescriptions = RANGE_TRAITS.reduce(
    (descriptions, trait) => mergeObject(descriptions, { [trait]: "PF2E.TraitDescriptionRange" }),
    {} as Record<typeof RANGE_TRAITS[number], string>
);

const preciousMaterialDescriptions = {
    abysium: "PF2E.PreciousMaterialAbysiumDescription",
    adamantine: "PF2E.PreciousMaterialAdamantineDescription",
    coldIron: "PF2E.PreciousMaterialColdIronDescription",
    darkwood: "PF2E.PreciousMaterialDarkwoodDescription",
    djezet: "PF2E.PreciousMaterialDjezetDescription",
    dragonhide: "PF2E.PreciousMaterialDragonhideDescription",
    "grisantian-pelt": "PF2E.PreciousMaterialGrisantianPeltDescription",
    inubrix: "PF2E.PreciousMaterialInubrixDescription",
    mithral: "PF2E.PreciousMaterialMithralDescription",
    noqual: "PF2E.PreciousMaterialNoqualDescription",
    orichalcum: "PF2E.PreciousMaterialOrichalcumDescription",
    siccatite: "PF2E.PreciousMaterialSiccatiteDescription",
    silver: "PF2E.PreciousMaterialSilverDescription",
    sovereignSteel: "PF2E.PreciousMaterialSovereignSteelDescription",
    warpglass: "PF2E.PreciousMaterialWarpglassDescription",
};

const traitsDescriptions = {
    aasimar: "PF2E.TraitDescriptionAasimar",
    aberration: "PF2E.TraitDescriptionAberration",
    abjuration: "PF2E.TraitDescriptionAbjuration",
    acid: "PF2E.TraitDescriptionAcid",
    additive1: "PF2E.TraitDescriptionAdditive",
    additive2: "PF2E.TraitDescriptionAdditive",
    additive3: "PF2E.TraitDescriptionAdditive",
    adjustment: "PF2E.TraitDescriptionAdjustment",
    aeon: "PF2E.TraitDescriptionAeon",
    aesir: "PF2E.TraitDescriptionAesir",
    aftermath: "PF2E.TraitDescriptionAftermath",
    agathion: "PF2E.TraitDescriptionAgathion",
    agile: "PF2E.TraitDescriptionAgile",
    air: "PF2E.TraitDescriptionAir",
    alchemical: "PF2E.TraitDescriptionAlchemical",
    alchemist: "PF2E.TraitDescriptionAlchemist",
    amphibious: "PF2E.TraitDescriptionAmphibious",
    anadi: "PF2E.TraitDescriptionAnadi",
    android: "PF2E.TraitDescriptionAndroid",
    apex: "PF2E.TraitDescriptionApex",
    aphorite: "PF2E.TraitDescriptionAphorite",
    arcane: "PF2E.TraitDescriptionArcane",
    archetype: "PF2E.TraitDescriptionArchetype",
    artifact: "PF2E.TraitDescriptionArtifact",
    attached: "PF2E.TraitDescriptionAttached",
    "attached-to-crossbow-or-firearm": "PF2E.TraitDescriptionAttached",
    "attached-to-shield": "PF2E.TraitDescriptionAttached",
    attack: "PF2E.TraitDescriptionAttack",
    auditory: "PF2E.TraitDescriptionAuditory",
    aura: "PF2E.TraitDescriptionAura",
    automaton: "PF2E.TraitDescriptionAutomaton",
    azarketi: "PF2E.TraitDescriptionAzarketi",
    backstabber: "PF2E.TraitDescriptionBackstabber",
    backswing: "PF2E.TraitDescriptionBackswing",
    barbarian: "PF2E.TraitDescriptionBarbarian",
    bard: "PF2E.TraitDescriptionBard",
    beastkin: "PF2E.TraitDescriptionBeastkin",
    bomb: "PF2E.TraitDescriptionBomb",
    brutal: "PF2E.TraitDescriptionBrutal",
    bulwark: "PF2E.TraitDescriptionBulwark",
    cantrip: "PF2E.TraitDescriptionCantrip",
    "capacity-3": "PF2E.TraitDescriptionCapacity",
    "capacity-4": "PF2E.TraitDescriptionCapacity",
    "capacity-5": "PF2E.TraitDescriptionCapacity",
    catalyst: "PF2E.TraitDescriptionCatalyst",
    catfolk: "PF2E.TraitDescriptionCatfolk",
    champion: "PF2E.TraitDescriptionChampion",
    changeling: "PF2E.TraitDescriptionChangeling",
    chaotic: "PF2E.TraitDescriptionChaotic",
    charm: "PF2E.TraitDescriptionCharm",
    class: "PF2E.TraitDescriptionClass",
    cleric: "PF2E.TraitDescriptionCleric",
    climbing: "PF2E.TraitDescriptionClimbing",
    clockwork: "PF2E.TraitDescriptionClockwork",
    cobbled: "PF2E.TraitDescriptionCobbled",
    cold: "PF2E.TraitDescriptionCold",
    combination: "PF2E.TraitDescriptionCombination",
    comfort: "PF2E.TraitDescriptionComfort",
    common: "PF2E.TraitDescriptionCommon",
    companion: "PF2E.TraitDescriptionCompanion",
    complex: "PF2E.TraitDescriptionComplex",
    composition: "PF2E.TraitDescriptionComposition",
    concealable: "PF2E.TraitDescriptionConcealable",
    concentrate: "PF2E.TraitDescriptionConcentrate",
    concussive: "PF2E.TraitDescriptionConcussive",
    conjuration: "PF2E.TraitDescriptionConjuration",
    conrasu: "PF2E.TraitDescriptionConrasu",
    consecration: "PF2E.TraitDescriptionConsecration",
    consumable: "PF2E.TraitDescriptionConsumable",
    contact: "PF2E.TraitDescriptionContact",
    contingency: "PF2E.TraitDescriptionContingency",
    contract: "PF2E.TraitDescriptionContract",
    "critical-fusion": "PF2E.TraitDescriptionCriticalFusion",
    curse: "PF2E.TraitDescriptionCurse",
    cursebound: "PF2E.TraitDescriptionCursebound",
    cursed: "PF2E.TraitDescriptionCursed",
    darkness: "PF2E.TraitDescriptionDarkness",
    darvakka: "PF2E.TraitDescriptionDarvakka",
    "deadly-2d10": "PF2E.TraitDescriptionDeadly",
    "deadly-2d12": "PF2E.TraitDescriptionDeadly",
    "deadly-2d8": "PF2E.TraitDescriptionDeadly",
    "deadly-3d10": "PF2E.TraitDescriptionDeadly",
    "deadly-3d12": "PF2E.TraitDescriptionDeadly",
    "deadly-3d8": "PF2E.TraitDescriptionDeadly",
    "deadly-4d10": "PF2E.TraitDescriptionDeadly",
    "deadly-4d12": "PF2E.TraitDescriptionDeadly",
    "deadly-4d8": "PF2E.TraitDescriptionDeadly",
    "deadly-d10": "PF2E.TraitDescriptionDeadly",
    "deadly-d12": "PF2E.TraitDescriptionDeadly",
    "deadly-d6": "PF2E.TraitDescriptionDeadly",
    "deadly-d8": "PF2E.TraitDescriptionDeadly",
    death: "PF2E.TraitDescriptionDeath",
    dedication: "PF2E.TraitDescriptionDedication",
    detection: "PF2E.TraitDescriptionDetection",
    deviant: "PF2E.TraitDescriptionDeviant",
    dhampir: "PF2E.TraitDescriptionDhampir",
    disarm: "PF2E.TraitDescriptionDisarm",
    disease: "PF2E.TraitDescriptionDisease",
    divination: "PF2E.TraitDescriptionDivination",
    divine: "PF2E.TraitDescriptionDivine",
    "double-barrel": "PF2E.TraitDescriptionDoubleBarrel",
    downtime: "PF2E.TraitDescriptionDowntime",
    drug: "PF2E.TraitDescriptionDrug",
    druid: "PF2E.TraitDescriptionDruid",
    duskwalker: "PF2E.TraitDescriptionDuskwalker",
    dwarf: "PF2E.TraitDescriptionDwarf",
    earth: "PF2E.TraitDescriptionEarth",
    eidolon: "PF2E.TraitDescriptionEidolon",
    electricity: "PF2E.TraitDescriptionElectricity",
    elf: "PF2E.TraitDescriptionElf",
    elixir: "PF2E.TraitDescriptionElixir",
    emotion: "PF2E.TraitDescriptionEmotion",
    enchantment: "PF2E.TraitDescriptionEnchantment",
    environment: "PF2E.TraitDescriptionEnvironment",
    esoterica: "PF2E.TraitDescriptionEsoterica",
    evil: "PF2E.TraitDescriptionEvil",
    evocation: "PF2E.TraitDescriptionEvocation",
    exploration: "PF2E.TraitDescriptionExploration",
    extradimensional: "PF2E.TraitDescriptionExtradimensional",
    "fatal-aim-d10": "PF2E.TraitDescriptionFatalAim",
    "fatal-aim-d12": "PF2E.TraitDescriptionFatalAim",
    "fatal-d10": "PF2E.TraitDescriptionFatal",
    "fatal-d12": "PF2E.TraitDescriptionFatal",
    "fatal-d8": "PF2E.TraitDescriptionFatal",
    fear: "PF2E.TraitDescriptionFear",
    fetchling: "PF2E.TraitDescriptionFetchling",
    fey: "PF2E.TraitDescriptionFey",
    fighter: "PF2E.TraitDescriptionFighter",
    finesse: "PF2E.TraitDescriptionFinesse",
    finisher: "PF2E.TraitDescriptionFinisher",
    fire: "PF2E.TraitDescriptionFire",
    fleshwarp: "PF2E.TraitDescriptionFleshwarp",
    flexible: "PF2E.TraitDescriptionFlexible",
    flourish: "PF2E.TraitDescriptionFlourish",
    focused: "PF2E.TraitDescriptionFocused",
    force: "PF2E.TraitDescriptionForce",
    forceful: "PF2E.TraitDescriptionForceful",
    fortune: "PF2E.TraitDescriptionFortune",
    "free-hand": "PF2E.TraitDescriptionFreeHand",
    fulu: "PF2E.TraitDescriptionFulu",
    fungus: "PF2E.TraitDescriptionFungus",
    gadget: "PF2E.TraitDescriptionGadget",
    ganzi: "PF2E.TraitDescriptionGanzi",
    general: "PF2E.TraitDescriptionGeneral",
    genie: "PF2E.TraitDescriptionGenie",
    geniekin: "PF2E.TraitDescriptionGeniekin",
    ghul: "PF2E.TraitDescriptionGhul",
    gnoll: "PF2E.TraitDescriptionGnoll",
    gnome: "PF2E.TraitDescriptionGnome",
    goblin: "PF2E.TraitDescriptionGoblin",
    goloma: "PF2E.TraitDescriptionGoloma",
    good: "PF2E.TraitDescriptionGood",
    grapple: "PF2E.TraitDescriptionGrapple",
    grimoire: "PF2E.TraitDescriptionGrimoire",
    grippli: "PF2E.TraitDescriptionGrippli",
    gunslinger: "PF2E.TraitDescriptionGunslinger",
    halfling: "PF2E.TraitDescriptionHalfling",
    "half-elf": "PF2E.TraitDescriptionHalfElf",
    "half-orc": "PF2E.TraitDescriptionHalfOrc",
    "jousting-d6": "PF2E.TraitDescriptionJoustingD6",
    hampering: "PF2E.TraitDescriptionHampering",
    haunt: "PF2E.TraitDescriptionHaunt",
    healing: "PF2E.TraitDescriptionHealing",
    hex: "PF2E.TraitDescriptionHex",
    hobgoblin: "PF2E.TraitDescriptionHobgoblin",
    human: "PF2E.TraitDescriptionHuman",
    humanoid: "PF2E.TraitDescriptionHumanoid",
    ifrit: "PF2E.TraitDescriptionIfrit",
    illusion: "PF2E.TraitDescriptionIllusion",
    incapacitation: "PF2E.TraitDescriptionIncapacitation",
    incarnate: "PF2E.TraitDescriptionIncarnate",
    incorporeal: "PF2E.TraitDescriptionIncorporeal",
    infused: "PF2E.TraitDescriptionInfused",
    ingested: "PF2E.TraitDescriptionIngested",
    inhaled: "PF2E.TraitDescriptionInhaled",
    injection: "PF2E.TraitDescriptionInjection",
    injury: "PF2E.TraitDescriptionInjury",
    intelligent: "PF2E.TraitDescriptionIntelligent",
    inventor: "PF2E.TraitDescriptionInventor",
    invested: "PF2E.TraitDescriptionInvested",
    investigator: "PF2E.TraitDescriptionInvestigator",
    kickback: "PF2E.TraitDescriptionKickback",
    kitsune: "TraitDescriptionKitsune",
    kobold: "TraitDescriptionKobold",
    lawful: "PF2E.TraitDescriptionLawful",
    leshy: "PF2E.TraitDescriptionLeshy",
    light: "PF2E.TraitDescriptionLight",
    linguistic: "PF2E.TraitDescriptionLinguistic",
    litany: "PF2E.TraitDescriptionLitany",
    lizardfolk: "PF2E.TraitDescriptionLizardfolk",
    magical: "PF2E.TraitDescriptionMagical",
    magus: "PF2E.TraitDescriptionMagus",
    manipulate: "PF2E.TraitDescriptionManipulate",
    mechanical: "PF2E.TraitDescriptionMechanical",
    mental: "PF2E.TraitDescriptionMental",
    metamagic: "PF2E.TraitDescriptionMetamagic",
    mindless: "PF2E.TraitDescriptionMindless",
    minion: "PF2E.TraitDescriptionMinion",
    misfortune: "PF2E.TraitDescriptionMisfortune",
    modification: "PF2E.TraitDescriptionModification",
    modular: "PF2E.TraitDescriptionModular",
    monk: "PF2E.TraitDescriptionMonk",
    morph: "PF2E.TraitDescriptionMorph",
    mounted: "PF2E.TraitDescriptionMounted",
    move: "PF2E.TraitDescriptionMove",
    multiclass: "PF2E.TraitDescriptionMulticlass",
    mutagen: "PF2E.TraitDescriptionMutagen",
    necromancy: "PF2E.TraitDescriptionNecromancy",
    negative: "PF2E.TraitDescriptionNegative",
    noisy: "PF2E.TraitDescriptionNoisy",
    nonlethal: "PF2E.TraitDescriptionNonlethal",
    oath: "PF2E.TraitDescriptionOath",
    occult: "PF2E.TraitDescriptionOccult",
    oil: "PF2E.TraitDescriptionOil",
    olfactory: "PF2E.TraitDescriptionOlfactory",
    open: "PF2E.TraitDescriptionOpen",
    oracle: "PF2E.TraitDescriptionOracle",
    orc: "PF2E.TraitDescriptionOrc",
    oread: "PF2E.TraitDescriptionOread",
    parry: "PF2E.TraitDescriptionParry",
    peachwood: "PF2E.PreciousMaterialPeachwoodDescription",
    plant: "PF2E.TraitDescriptionPlant",
    poison: "PF2E.TraitDescriptionPoison",
    polymorph: "PF2E.TraitDescriptionPolymorph",
    poppet: "PF2E.TraitDescriptionPoppet",
    portable: "PF2E.TraitDescriptionPortable",
    positive: "PF2E.TraitDescriptionPositive",
    possession: "PF2E.TraitDescriptionPossession",
    potion: "PF2E.TraitDescriptionPotion",
    precious: "PF2E.TraitDescriptionPrecious",
    prediction: "PF2E.TraitDescriptionPrediction",
    press: "PF2E.TraitDescriptionPress",
    primal: "PF2E.TraitDescriptionPrimal",
    propulsive: "PF2E.TraitDescriptionPropulsive",
    psychic: "PF2E.TraitDescriptionPsychic",
    rage: "PF2E.TraitDescriptionRage",
    ranger: "PF2E.TraitDescriptionRanger",
    ...rangeDescriptions,
    "ranged-trip": "PF2E.TraitDescriptionRangedTrip",
    rare: "PF2E.TraitDescriptionRare",
    ratfolk: "PF2E.TraitDescriptionRatfolk",
    reach: "PF2E.TraitDescriptionReach",
    "reach-0": "PF2E.TraitDescriptionReach",
    "reach-10": "PF2E.TraitDescriptionReach",
    "reach-100": "PF2E.TraitDescriptionReach",
    "reach-1000": "PF2E.TraitDescriptionReach",
    "reach-15": "PF2E.TraitDescriptionReach",
    "reach-20": "PF2E.TraitDescriptionReach",
    "reach-25": "PF2E.TraitDescriptionReach",
    "reach-30": "PF2E.TraitDescriptionReach",
    "reach-40": "PF2E.TraitDescriptionReach",
    "reach-50": "PF2E.TraitDescriptionReach",
    "reach-60": "PF2E.TraitDescriptionReach",
    reflection: "PF2E.TraitDescriptionReflection",
    reload: "PF2E.TraitDescriptionReload",
    "reload-0": "PF2E.TraitDescriptionReload",
    "reload-1": "PF2E.TraitDescriptionReload",
    "reload-1-min": "PF2E.TraitDescriptionReload",
    "reload-2": "PF2E.TraitDescriptionReload",
    repeating: "PF2E.TraitDescriptionRepeating",
    resonant: "PF2E.TraitDescriptionResonant",
    revelation: "PF2E.TraitDescriptionRevelation",
    rogue: "PF2E.TraitDescriptionRogue",
    saggorak: "PF2E.TraitDescriptionSaggorak",
    "scatter-10": "PF2E.TraitDescriptionScatter",
    "scatter-15": "PF2E.TraitDescriptionScatter",
    "scatter-5": "PF2E.TraitDescriptionScatter",
    scroll: "PF2E.TraitDescriptionScroll",
    scrying: "PF2E.TraitDescriptionScrying",
    secret: "PF2E.TraitDescriptionSecret",
    shadow: "PF2E.TraitDescriptionShadow",
    shisk: "PF2E.TraitDescriptionShisk",
    shoony: "PF2E.TraitDescriptionShoony",
    shove: "PF2E.TraitDescriptionShove",
    skeleton: "PF2E.TraitDescriptionSkeleton",
    skill: "PF2E.TraitDescriptionSkill",
    sleep: "PF2E.TraitDescriptionSleep",
    snare: "PF2E.TraitDescriptionSnare",
    social: "PF2E.TraitDescriptionSocial",
    sonic: "PF2E.TraitDescriptionSonic",
    sorcerer: "PF2E.TraitDescriptionSorcerer",
    spellheart: "PF2E.TraitDescriptionSpellheart",
    spellshot: "PF2E.TraitNoDescription",
    splash: "PF2E.TraitDescriptionSplash",
    "splash-10": "PF2E.TraitDescriptionSplash10",
    sprite: "PF2E.TraitDescriptionSprite",
    staff: "PF2E.TraitDescriptionStaff",
    stance: "PF2E.TraitDescriptionStance",
    steam: "PF2E.TraitDescriptionSteam",
    strix: "PF2E.TraitDescriptionStrix",
    structure: "PF2E.TraitDescriptionStructure",
    suli: "PF2E.TraitDescriptionSuli",
    summon: "PF2E.TraitDescriptionSummon",
    summoned: "PF2E.TraitDescriptionSummoned",
    summoner: "PF2E.TraitDescriptionSummoner",
    swashbuckler: "PF2E.TraitDescriptionSwashbuckler",
    sweep: "PF2E.TraitDescriptionSweep",
    sylph: "PF2E.TraitDescriptionSylph",
    talisman: "PF2E.TraitDescriptionTalisman",
    tattoo: "PF2E.TraitDescriptionTattoo",
    telepathy: "PF2E.TraitDescriptionTelepathy",
    teleportation: "PF2E.TraitDescriptionTeleportation",
    tengu: "PF2E.TraitDescriptionTengu",
    tethered: "PF2E.TraitDescriptionTethered",
    thaumaturge: "PF2E.TraitDescriptionThaumaturge",
    thrown: "PF2E.TraitDescriptionThrown",
    "thrown-10": "PF2E.TraitDescriptionThrown",
    "thrown-100": "PF2E.TraitDescriptionThrown",
    "thrown-15": "PF2E.TraitDescriptionThrown",
    "thrown-20": "PF2E.TraitDescriptionThrown",
    "thrown-25": "PF2E.TraitDescriptionThrown",
    "thrown-30": "PF2E.TraitDescriptionThrown",
    "thrown-40": "PF2E.TraitDescriptionThrown",
    "thrown-50": "PF2E.TraitDescriptionThrown",
    "thrown-60": "PF2E.TraitDescriptionThrown",
    "thrown-80": "PF2E.TraitDescriptionThrown",
    tiefling: "PF2E.TraitDescriptionTiefling",
    time: "PF2E.TraitDescriptionTime",
    transmutation: "PF2E.TraitDescriptionTransmutation",
    trap: "PF2E.TraitDescriptionTrap",
    trip: "PF2E.TraitDescriptionTrip",
    "true-name": "PF2E.TraitDescriptionTrueName",
    twin: "PF2E.TraitDescriptionTwin",
    "two-hand-d10": "PF2E.TraitDescriptionTwoHand",
    "two-hand-d12": "PF2E.TraitDescriptionTwoHand",
    "two-hand-d6": "PF2E.TraitDescriptionTwoHand",
    "two-hand-d8": "PF2E.TraitDescriptionTwoHand",
    unarmed: "PF2E.TraitDescriptionUnarmed",
    uncommon: "PF2E.TraitDescriptionUncommon",
    undine: "PF2E.TraitDescriptionUndine",
    unique: "PF2E.TraitDescriptionUnique",
    unstable: "PF2E.TraitDescriptionUnstable",
    "versatile-acid": "PF2E.TraitDescriptionVersatile",
    "versatile-b": "PF2E.TraitDescriptionVersatile",
    "versatile-chaotic": "PF2E.TraitDescriptionVersatile",
    "versatile-cold": "PF2E.TraitDescriptionVersatile",
    "versatile-electricity": "PF2E.TraitDescriptionVersatile",
    "versatile-evil": "PF2E.TraitDescriptionVersatile",
    "versatile-fire": "PF2E.TraitDescriptionVersatile",
    "versatile-force": "PF2E.TraitDescriptionVersatile",
    "versatile-good": "PF2E.TraitDescriptionVersatile",
    "versatile-lawful": "PF2E.TraitDescriptionVersatile",
    "versatile-negative": "PF2E.TraitDescriptionVersatile",
    "versatile-p": "PF2E.TraitDescriptionVersatile",
    "versatile-positive": "PF2E.TraitDescriptionVersatile",
    "versatile-s": "PF2E.TraitDescriptionVersatile",
    "versatile-sonic": "PF2E.TraitDescriptionVersatile",
    virulent: "PF2E.TraitDescriptionVirulent",
    visual: "PF2E.TraitDescriptionVisual",
    "volley-20": "PF2E.TraitDescriptionVolley",
    "volley-30": "PF2E.TraitDescriptionVolley",
    "volley-50": "PF2E.TraitDescriptionVolley",
    wand: "PF2E.TraitDescriptionWand",
    water: "PF2E.TraitDescriptionWater",
    witch: "PF2E.TraitDescriptionWitch",
    wizard: "PF2E.TraitDescriptionWizard",
    ...preciousMaterialDescriptions,
};

const preciousMaterialGrades: Record<PreciousMaterialGrade, string> = {
    low: "PF2E.PreciousMaterialLowGrade",
    standard: "PF2E.PreciousMaterialStandardGrade",
    high: "PF2E.PreciousMaterialHighGrade",
};

const meleeWeaponGroups: Record<MeleeWeaponGroup, string> = {
    axe: "PF2E.WeaponGroupAxe",
    brawling: "PF2E.WeaponGroupBrawling",
    club: "PF2E.WeaponGroupClub",
    dart: "PF2E.WeaponGroupDart",
    flail: "PF2E.WeaponGroupFlail",
    hammer: "PF2E.WeaponGroupHammer",
    knife: "PF2E.WeaponGroupKnife",
    pick: "PF2E.WeaponGroupPick",
    polearm: "PF2E.WeaponGroupPolearm",
    shield: "PF2E.WeaponGroupShield",
    spear: "PF2E.WeaponGroupSpear",
    sword: "PF2E.WeaponGroupSword",
};

const weaponGroups: Record<WeaponGroup, string> = {
    ...meleeWeaponGroups,
    bomb: "PF2E.WeaponGroupBomb",
    bow: "PF2E.WeaponGroupBow",
    firearm: "PF2E.WeaponGroupFirearm",
    sling: "PF2E.WeaponGroupSling",
};

const weaponPropertyRunes = {
    ...Object.entries(WEAPON_PROPERTY_RUNES).reduce((accumulated, [slug, rune]) => {
        return { ...accumulated, [slug]: rune.name };
    }, {} as Record<WeaponPropertyRuneType, string>),
};

// Creature and Equipment Sizes
const sizeTypes: Record<Size, string> = {
    tiny: "PF2E.ActorSizeTiny",
    sm: "PF2E.ActorSizeSmall",
    med: "PF2E.ActorSizeMedium",
    lg: "PF2E.ActorSizeLarge",
    huge: "PF2E.ActorSizeHuge",
    grg: "PF2E.ActorSizeGargantuan",
};

const featTypes: Record<FeatType, string> = {
    ancestry: "PF2E.FeatTypeAncestry",
    ancestryfeature: "PF2E.FeatTypeAncestryfeature",
    class: "PF2E.FeatTypeClass",
    classfeature: "PF2E.FeatTypeClassfeature",
    skill: "PF2E.FeatTypeSkill",
    general: "PF2E.FeatTypeGeneral",
    archetype: "PF2E.FeatTypeArchetype",
    bonus: "PF2E.FeatTypeBonus",
    pfsboon: "PF2E.FeatPFSBoonHeader",
    deityboon: "PF2E.FeatDeityBoonHeader",
    curse: "PF2E.FeatCurseHeader",
};

const alignments: Record<Alignment, string> = {
    LG: "PF2E.AlignmentLG",
    NG: "PF2E.AlignmentNG",
    CG: "PF2E.AlignmentCG",
    LN: "PF2E.AlignmentLN",
    N: "PF2E.AlignmentN",
    CN: "PF2E.AlignmentCN",
    LE: "PF2E.AlignmentLE",
    NE: "PF2E.AlignmentNE",
    CE: "PF2E.AlignmentCE",
};

const deityDomains = Object.keys(enJSON.PF2E.Item.Deity.Domain).reduce((domains, key) => {
    const slug = sluggify(key);
    const casedKey = sluggify(key, { camel: "bactrian" });
    return {
        ...domains,
        [slug]: {
            label: `PF2E.Item.Deity.Domain.${casedKey}.Label`,
            description: `PF2E.Item.Deity.Domain.${casedKey}.Description`,
        },
    };
}, {} as Record<DeityDomain, { label: string; description: string }>);

const weaponReload: Record<WeaponReloadTime, string> = {
    "-": "—", // Reload value for thrown weapons
    0: "0",
    1: "1",
    2: "2",
    3: "3",
    10: "PF2E.Item.Weapon.ReloadOneMinute",
};

export const PF2ECONFIG = {
    chatDamageButtonShieldToggle: false,

    statusEffects: {
        lastIconTheme: "default" as StatusEffectIconTheme,
        iconDir: "systems/pf2e/icons/conditions/",
        conditions: tokenHUDConditions,
    },

    levels: {
        1: "PF2E.Level1",
        2: "PF2E.Level2",
        3: "PF2E.Level3",
        4: "PF2E.Level4",
        5: "PF2E.Level5",
        6: "PF2E.Level6",
        7: "PF2E.Level7",
        8: "PF2E.Level8",
        9: "PF2E.Level9",
        10: "PF2E.Level10",
        11: "PF2E.Level11",
        12: "PF2E.Level12",
        13: "PF2E.Level13",
        14: "PF2E.Level14",
        15: "PF2E.Level15",
        16: "PF2E.Level16",
        17: "PF2E.Level17",
        18: "PF2E.Level18",
        19: "PF2E.Level19",
        20: "PF2E.Level20",
    },

    abilities: {
        str: "PF2E.AbilityStr",
        dex: "PF2E.AbilityDex",
        con: "PF2E.AbilityCon",
        int: "PF2E.AbilityInt",
        wis: "PF2E.AbilityWis",
        cha: "PF2E.AbilityCha",
    },

    attributes: {
        perception: "PF2E.PerceptionLabel",
        stealth: "PF2E.StealthLabel",
        initiative: "PF2E.PerceptionLabel",
    },

    dcAdjustments: {
        "incredibly easy": "PF2E.DCAdjustmentIncrediblyEasy",
        "very easy": "PF2E.DCAdjustmentVeryEasy",
        easy: "PF2E.DCAdjustmentEasy",
        normal: "PF2E.DCAdjustmentNormal",
        hard: "PF2E.DCAdjustmentHard",
        "very hard": "PF2E.DCAdjustmentVeryHard",
        "incredibly hard": "PF2E.DCAdjustmentIncrediblyHard",
    },

    skills: {
        acr: "PF2E.SkillAcr",
        arc: "PF2E.SkillArc",
        ath: "PF2E.SkillAth",
        cra: "PF2E.SkillCra",
        dec: "PF2E.SkillDec",
        dip: "PF2E.SkillDip",
        itm: "PF2E.SkillItm",
        med: "PF2E.SkillMed",
        nat: "PF2E.SkillNat",
        occ: "PF2E.SkillOcc",
        prf: "PF2E.SkillPrf",
        rel: "PF2E.SkillRel",
        soc: "PF2E.SkillSoc",
        ste: "PF2E.SkillSte",
        sur: "PF2E.SkillSur",
        thi: "PF2E.SkillThi",
    },

    martialSkills: {
        unarmored: "PF2E.MartialUnarmored",
        light: "PF2E.MartialLight",
        medium: "PF2E.MartialMedium",
        heavy: "PF2E.MartialHeavy",
        simple: "PF2E.MartialSimple",
        martial: "PF2E.MartialMartial",
        advanced: "PF2E.MartialAdvanced",
        unarmed: "PF2E.MartialUnarmed",
    },

    saves: {
        fortitude: "PF2E.SavesFortitude",
        reflex: "PF2E.SavesReflex",
        will: "PF2E.SavesWill",
    },

    savingThrowDefaultAbilities: {
        fortitude: "con",
        reflex: "dex",
        will: "wis",
    } as const,

    currencies: {
        pp: "PF2E.CurrencyPP",
        gp: "PF2E.CurrencyGP",
        sp: "PF2E.CurrencySP",
        cp: "PF2E.CurrencyCP",
    },

    preciousMaterialGrades,
    preciousMaterials,

    armorPotencyRunes: {
        "1": "PF2E.ArmorPotencyRune1",
        "2": "PF2E.ArmorPotencyRune2",
        "3": "PF2E.ArmorPotencyRune3",
        "4": "PF2E.ArmorPotencyRune4",
    },

    armorResiliencyRunes: {
        resilient: "PF2E.ArmorResilientRune",
        greaterResilient: "PF2E.ArmorGreaterResilientRune",
        majorResilient: "PF2E.ArmorMajorResilientRune",
    },
    armorPropertyRunes: {
        ready: "PF2E.ArmorPropertyRuneReady",
        slick: "PF2E.ArmorPropertyRuneSlick",
        shadow: "PF2E.ArmorPropertyRuneShadow",
        glamered: "PF2E.ArmorPropertyRuneGlamered",
        acidResistant: "PF2E.ArmorPropertyRuneAcidResistant",
        coldResistant: "PF2E.ArmorPropertyRuneColdResistant",
        electricityResistant: "PF2E.ArmorPropertyRuneElectricityResistant",
        fireResistant: "PF2E.ArmorPropertyRuneFireResistant",
        greaterSlick: "PF2E.ArmorPropertyRuneGreaterSlick",
        invisibility: "PF2E.ArmorPropertyRuneInvisibility",
        sinisterKnight: "PF2E.ArmorPropertyRuneSinisterKnight",
        greaterReady: "PF2E.ArmorPropertyRuneGreaterReady",
        greaterShadow: "PF2E.ArmorPropertyRuneGreaterShadow",
        greaterInvisibility: "PF2E.ArmorPropertyRuneGreaterInvisibility",
        greaterAcidResistant: "PF2E.ArmorPropertyRuneGreaterAcidResistant",
        greaterColdResistant: "PF2E.ArmorPropertyRuneGreaterColdResistant",
        greaterElectricityResistant: "PF2E.ArmorPropertyRuneGreaterElectricityResistant",
        greaterFireResistant: "PF2E.ArmorPropertyRuneGreaterFireResistant",
        fortification: "PF2E.ArmorPropertyRuneFortification",
        winged: "PF2E.ArmorPropertyRuneWinged",
        rockBraced: "PF2E.ArmorPropertyRuneRockBraced",
        soaring: "PF2E.ArmorPropertyRuneSoaring",
        antimagic: "PF2E.ArmorPropertyRuneAntimagic",
        majorSlick: "PF2E.ArmorPropertyRuneMajorSlick",
        ethereal: "PF2E.ArmorPropertyRuneEthereal",
        majorShadow: "PF2E.ArmorPropertyRuneMajorShadow",
        greaterFortification: "PF2E.ArmorPropertyRuneGreaterFortification",
        greaterWinged: "PF2E.ArmorPropertyRuneGreaterWinged",
        deathless: "PF2E.ArmorPropertyRuneDeathless",
        dread: "PF2E.ArmorPropertyRuneDread",
        bitter: "PF2E.ArmorPropertyRuneBitter",
        stanching: "PF2E.ArmorPropertyRuneStanching",
        greaterStanching: "PF2E.ArmorPropertyRuneGreaterStanching",
        majorStanching: "PF2E.ArmorPropertyRuneMajorStanching",
        trueStanching: "PF2E.ArmorPropertyRuneTrueStanching",
        implacable: "PF2E.ArmorPropertyRuneImplacable",
    },
    accessoryPropertyRunes: {
        called: "PF2E.AccessoryPropertyRuneCalled",
        dragonsBreath: "PF2E.AccessoryPropertyRuneDragonsBreath",
        paired: "PF2E.AccessoryPropertyRunePaired",
        greaterPaired: "PF2E.AccessoryPropertyRuneGreaterPaired",
        majorPaired: "PF2E.AccessoryPropertyRuneMajorPaired",
        presentable: "PF2E.AccessoryPropertyRunePresentable",
        snagging: "PF2E.AccessoryPropertyRuneSnagging",
        softLanding: "PF2E.AccessoryPropertyRuneSoftLanding",
        spellBastion: "PF2E.AccessoryPropertyRuneSpellBastion",
        windCatcher: "PF2E.AccessoryPropertyRuneWindCatcher",
        greaterWindCatcher: "PF2E.AccessoryPropertyRuneGreaterWindCatcher",
    },
    weaponPotencyRunes: {
        1: "PF2E.WeaponPotencyRune1",
        2: "PF2E.WeaponPotencyRune2",
        3: "PF2E.WeaponPotencyRune3",
        4: "PF2E.WeaponPotencyRune4",
    },
    weaponStrikingRunes: {
        striking: "PF2E.ArmorStrikingRune",
        greaterStriking: "PF2E.ArmorGreaterStrikingRune",
        majorStriking: "PF2E.ArmorMajorStrikingRune",
    },
    weaponPropertyRunes,
    damageTraits,
    damageTypes,
    damageSubtypes: {
        persistent: "PF2E.ConditionTypePersistentShort",
        splash: "PF2E.TraitSplash",
    },

    damageCategories,
    resistanceTypes,

    stackGroups: {
        arrows: "PF2E.StackGroupArrows",
        bolts: "PF2E.StackGroupBolts",
        slingBullets: "PF2E.StackGroupSlingBullets",
        blowgunDarts: "PF2E.StackGroupBlowgunDarts",
        woodenTaws: "PF2E.StackGroupWoodenTaws",
        rounds5: "PF2E.StackGroupRounds5",
        rounds10: "PF2E.StackGroupRounds10",
        rations: "PF2E.StackGroupRations",
        coins: "PF2E.StackGroupCoins",
        gems: "PF2E.StackGroupGems",
        sacks: "PF2E.StackGroupSacks",
    },

    weaknessTypes,

    weaponDamage: {
        bludgeoning: "PF2E.TraitBludgeoning",
        piercing: "PF2E.TraitPiercing",
        slashing: "PF2E.TraitSlashing",
        modular: "PF2E.TraitModular",
    },

    healingTypes: {
        healing: "PF2E.TraitHealing",
        temphp: "PF2E.HealingTypeTemporaryHealing",
    },

    weaponCategories,
    weaponGroups,
    meleeWeaponGroups,

    baseWeaponTypes,
    equivalentWeapons,

    weaponDescriptions: {
        club: "PF2E.WeaponDescriptionClub",
        knife: "PF2E.WeaponDescriptionKnife",
        brawling: "PF2E.WeaponDescriptionBrawling",
        spear: "PF2E.WeaponDescriptionSpear",
        sword: "PF2E.WeaponDescriptionSword",
        axe: "PF2E.WeaponDescriptionAxe",
        flail: "PF2E.WeaponDescriptionFlail",
        polearm: "PF2E.WeaponDescriptionPolearm",
        pick: "PF2E.WeaponDescriptionPick",
        hammer: "PF2E.WeaponDescriptionHammer",
        shield: "PF2E.WeaponDescriptionShield",
        dart: "PF2E.WeaponDescriptionDart",
        bow: "PF2E.WeaponDescriptionBow",
        sling: "PF2E.WeaponDescriptionSling",
        bomb: "PF2E.WeaponDescriptionBomb",
    },

    usageTraits: {
        "affixed-to-a-ranged-weapon": "PF2E.TraitAffixedToARangedWeapon",
        "affixed-to-a-shield": "PF2E.TraitAffixedToAShield",
        "affixed-to-a-two-handed-firearm-or-crossbow": "PF2E.TraitAffixededToATwoHandedFirearmOrCrossbow",
        "affixed-to-armor": "PF2E.TraitAffixedToArmor",
        "affixed-to-medium-heavy-armor": "PF2E.TraitAffixedToMediumHeavyArmor",
        "affixed-to-armor-or-a-weapon": "PF2E.TraitAffixedToArmorOrAWeapon",
        "affixed-to-armor-or-travelers-clothing": "PF2E.TraitAffixedToArmorOrTravelersClothing",
        "affixed-to-crossbow-or-firearm": "PF2E.TraitAffixededToCrossbowOrFirearm",
        "affixed-to-firearm": "PF2E.TraitAffixededToFirearm",
        "affixed-to-firearm-with-a-reload-of-1": "PF2E.TraitAffixededToFirearmWithAReloadOf1",
        "affixed-to-firearm-with-the-kickback-trait": "PF2E.TraitAffixededToFirearmWithTheKickbackTrait",
        "affixed-to-headgear": "PF2E.TraitAffixedToHeadgear",
        "affixed-to-weapon": "PF2E.TraitAffixedToWeapon",
        "applied-to-a-wind-powered-vehicle": "PF2E.TraitAppliedToAWindPoweredVehicle",
        "applied-to-any-item-of-light-or-negligible-bulk": "PF2E.TraitAppliedToAnyItemOfLightOrNegligibleBulk",
        "applied-to-any-visible-article-of-clothing": "PF2E.TraitAppliedToAnyVisibleArticleOfClothing",
        "applied-to-belt-cape-cloak-or-scarf": "PF2E.TraitAppliedToBeltCapeCloakOrScarf",
        "applied-to-boots-cape-cloak-or-umbrella": "PF2E.TraitAppliedToBootsCapeCloakOrUmbrella",
        "applied-to-dueling-cape-or-shield": "PF2E.TraitAppliedToDuelingCapeOrShield",
        "applied-to-buckler-shield": "PF2E.TraitAppliedToBucklerShield",
        "applied-to-shield": "PF2E.TraitAppliedToShield",
        "attached-to-a-thrown-weapon": "PF2E.TraitAttachedToAThrownWeapon",
        "attached-to-crossbow-or-firearm": "PF2E.TraitAttachedToCrossbowOrFirearm",
        "attached-to-crossbow-or-firearm-firing-mechanism": "PF2E.TraitAttachedToCrossbowOrFirearmFiringMechanism",
        "attached-to-crossbow-or-firearm-scope": "PF2E.TraitAttachedToCrossbowOrFirearmScope",
        "attached-to-firearm": "PF2E.TraitAttachedToFirearm",
        "attached-to-firearm-scope": "PF2E.TraitAttachedToFirearmScope",
        bonded: "PF2E.TraitBonded",
        "each-rune-applied-to-a-separate-item-that-has-pockets":
            "PF2E.TraitEachRuneAppliedToASeparateItemThatHasPockets",
        "etched-onto-a-weapon": "PF2E.TraitEtchedOntoAWeapon",
        "etched-onto-armor": "PF2E.TraitEtchedOntoArmor",
        "etched-onto-clan-dagger": "PF2E.TraitEtchedOntoAClanDagger",
        "etched-onto-lm-nonmetal-armor": "PF2E.TraitEtchedOntoLightMedNMArmor",
        "etched-onto-med-heavy-armor": "PF2E.TraitEtchedOntoMedHeavyArmor",
        "etched-onto-melee-weapon": "PF2E.TraitEtchedOntoAMeleeWeapon",
        "etched-onto-thrown-weapon": "PF2E.TraitEtchedOntoAThrownWeapon",
        "held-in-one-hand": "PF2E.TraitHeldOneHand",
        "held-in-two-hands": "PF2E.TraitHeldTwoHands",
        other: "Other",
        "sewn-into-clothing": "PF2E.TraitSewnIntoClothing",
        "tattooed-on-the-body": "PF2E.TraitTattooedOnTheBody",
        worn: "PF2E.TraitWorn",
        wornamulet: "PF2E.TraitWornAmulet",
        wornanklets: "PF2E.TraitWornAnklets",
        wornarmbands: "PF2E.TraitWornArmbands",
        wornbackpack: "PF2E.TraitWornBackpack",
        wornbarding: "PF2E.TraitWornBarding",
        wornbelt: "PF2E.TraitWornBelt",
        wornbeltpouch: "PF2E.TraitWornBeltPouch",
        wornbracelet: "PF2E.TraitWornBracelet",
        wornbracers: "PF2E.TraitWornBracers",
        worncirclet: "PF2E.TraitWornCirclet",
        worncloak: "PF2E.TraitWornCloak",
        wornclothing: "PF2E.TraitWornClothing",
        worncollar: "PF2E.TraitWornCollar",
        worncrown: "PF2E.TraitWornCrown",
        wornepaulet: "PF2E.TraitWornEpaulet",
        worneyepiece: "PF2E.TraitWornEyepiece",
        worngarment: "PF2E.TraitWornGarment",
        worngloves: "PF2E.TraitWornGloves",
        wornheadwear: "PF2E.TraitWornHeadwear",
        wornhorseshoes: "PF2E.TraitWornHorseshoes",
        wornmask: "PF2E.TraitWornMask",
        wornnecklace: "PF2E.TraitWornNecklace",
        wornonbelt: "PF2E.TraitWornOnBelt",
        wornring: "PF2E.TraitWornRing",
        wornsaddle: "PF2E.TraitWornSaddle",
        wornshoes: "PF2E.TraitWornShoes",
        wornwrist: "PF2E.TraitWornOnWrists",
        "worn-and-attached-to-two-weapons": "PF2E.TraitWornAndAttachedToTwoWeapons",
        "worn-under-armor": "PF2E.TraitWornUnderArmor",
    },

    rarityTraits: {
        common: "PF2E.TraitCommon",
        uncommon: "PF2E.TraitUncommon",
        rare: "PF2E.TraitRare",
        unique: "PF2E.TraitUnique",
    },

    magicTraditions,
    spellOtherTraits,

    magicSchools,
    classTraits,
    ancestryTraits,
    ancestryItemTraits,
    deityDomains,

    weaponTraits,
    otherWeaponTags,

    armorTraits,
    otherArmorTags,

    equipmentTraits,
    otherEquipmentTags,

    actionTraits,
    consumableTraits,
    spellTraits,
    featTraits,
    creatureTraits,
    monsterTraits: creatureTraits,
    npcAttackTraits,
    hazardTraits,
    vehicleTraits,

    traitsDescriptions,

    weaponHands: {
        1: "PF2E.WeaponHands1",
        "1+": "PF2E.WeaponHands1Plus",
        2: "PF2E.WeaponHands2",
    },

    itemBonuses: {
        "-2": "PF2E.ItemBonusMinus2",
        0: "PF2E.ItemBonus0",
        1: "PF2E.ItemBonus1",
        2: "PF2E.ItemBonus2",
        3: "PF2E.ItemBonus3",
    },

    damageDice: {
        0: "0",
        1: "1",
        2: "2",
        3: "3",
        4: "4",
    },

    damageDie: {
        d4: "PF2E.DamageDieD4",
        d6: "PF2E.DamageDieD6",
        d8: "PF2E.DamageDieD8",
        d10: "PF2E.DamageDieD10",
        d12: "PF2E.DamageDieD12",
    },

    weaponMAP: {
        1: "-1/-2",
        2: "-2/-4",
        3: "-3/-6",
        4: "-4/-8",
        5: "-5/-10",
    },

    weaponReload,

    armorTypes: {
        unarmored: "PF2E.ArmorTypeUnarmored",
        light: "PF2E.ArmorTypeLight",
        medium: "PF2E.ArmorTypeMedium",
        heavy: "PF2E.ArmorTypeHeavy",
        shield: "PF2E.ArmorTypeShield",
    },

    armorGroups: {
        composite: "PF2E.ArmorGroupComposite",
        chain: "PF2E.ArmorGroupChain",
        cloth: "PF2E.ArmorGroupCloth",
        leather: "PF2E.ArmorGroupLeather",
        plate: "PF2E.ArmorGroupPlate",
    },

    consumableTypes: {
        ammo: "PF2E.ConsumableTypeAmmo",
        catalyst: "PF2E.TraitCatalyst",
        drug: "PF2E.ConsumableTypeDrug",
        elixir: "PF2E.ConsumableTypeElixir",
        fulu: "PF2E.TraitFulu",
        gadget: "PF2E.TraitGadget",
        oil: "PF2E.ConsumableTypeOil",
        other: "PF2E.ConsumableTypeOther",
        mutagen: "PF2E.ConsumableTypeMutagen",
        poison: "PF2E.ConsumableTypePoison",
        potion: "PF2E.ConsumableTypePotion",
        scroll: "PF2E.ConsumableTypeScroll",
        snare: "PF2E.ConsumableTypeSnare",
        talisman: "PF2E.ConsumableTypeTalisman",
        tool: "PF2E.ConsumableTypeTool",
        wand: "PF2E.ConsumableTypeWand",
    },

    preparationType: {
        prepared: "PF2E.PreparationTypePrepared",
        spontaneous: "PF2E.PreparationTypeSpontaneous",
        innate: "PF2E.PreparationTypeInnate",
        focus: "PF2E.SpellCategoryFocus",
        ritual: "PF2E.SpellCategoryRitual",
    },

    areaTypes: {
        burst: "PF2E.AreaTypeBurst",
        cone: "PF2E.AreaTypeCone",
        cube: "PF2E.AreaTypeCube",
        emanation: "PF2E.AreaTypeEmanation",
        line: "PF2E.AreaTypeLine",
        square: "PF2E.AreaTypeSquare",
    },

    areaSizes: {
        5: "PF2E.AreaSize5",
        10: "PF2E.AreaSize10",
        15: "PF2E.AreaSize15",
        20: "PF2E.AreaSize20",
        25: "PF2E.AreaSize25",
        30: "PF2E.AreaSize30",
        40: "PF2E.AreaSize40",
        45: "PF2E.AreaSize45",
        50: "PF2E.AreaSize50",
        60: "PF2E.AreaSize60",
        75: "PF2E.AreaSize75",
        80: "PF2E.AreaSize80",
        90: "PF2E.AreaSize90",
        100: "PF2E.AreaSize100",
        120: "PF2E.AreaSize120",
        360: "PF2E.AreaSize360",
        500: "PF2E.AreaSize500",
        1000: "PF2E.AreaSize1000",
        1320: "PF2E.AreaSizeQuarterMile",
        5280: "PF2E.AreaSize1Mile",
    },

    alignments,
    alignmentTraits,

    attitude: {
        hostile: "PF2E.Attitudes.Hostile",
        unfriendly: "PF2E.Attitudes.Unfriendly",
        indifferent: "PF2E.Attitudes.Indifferent",
        friendly: "PF2E.Attitudes.Friendly",
        helpful: "PF2E.Attitudes.Helpful",
    },

    skillList: {
        acrobatics: "PF2E.SkillAcrobatics",
        arcana: "PF2E.SkillArcana",
        athletics: "PF2E.SkillAthletics",
        crafting: "PF2E.SkillCrafting",
        deception: "PF2E.SkillDeception",
        diplomacy: "PF2E.SkillDiplomacy",
        intimidation: "PF2E.SkillIntimidation",
        medicine: "PF2E.SkillMedicine",
        nature: "PF2E.SkillNature",
        occultism: "PF2E.SkillOccultism",
        performance: "PF2E.SkillPerformance",
        religion: "PF2E.SkillReligion",
        society: "PF2E.SkillSociety",
        stealth: "PF2E.SkillStealth",
        survival: "PF2E.SkillSurvival",
        thievery: "PF2E.SkillThievery",
        lore: "PF2E.SkillLore",
    },

    spellComponents: {
        V: "PF2E.SpellComponentV",
        S: "PF2E.SpellComponentS",
        M: "PF2E.SpellComponentM",
        F: "PF2E.SpellComponentF",
    },

    spellCategories: {
        spell: "PF2E.SpellCategorySpell",
        focus: "PF2E.SpellCategoryFocus",
        ritual: "PF2E.SpellCategoryRitual",
    },

    spellTypes: {
        attack: "PF2E.SpellTypeAttack",
        save: "PF2E.SpellTypeSave",
        heal: "PF2E.SpellTypeHeal",
        utility: "PF2E.SpellTypeUtility",
    },

    spellLevels: {
        1: "PF2E.SpellLevel1",
        2: "PF2E.SpellLevel2",
        3: "PF2E.SpellLevel3",
        4: "PF2E.SpellLevel4",
        5: "PF2E.SpellLevel5",
        6: "PF2E.SpellLevel6",
        7: "PF2E.SpellLevel7",
        8: "PF2E.SpellLevel8",
        9: "PF2E.SpellLevel9",
        10: "PF2E.SpellLevel10",
    }, // TODO: Compute levels!

    featTypes,

    actionTypes: {
        action: "PF2E.ActionTypeAction",
        reaction: "PF2E.ActionTypeReaction",
        free: "PF2E.ActionTypeFree",
        passive: "PF2E.ActionTypePassive",
    },

    actionsNumber: {
        1: "PF2E.ActionNumber1",
        2: "PF2E.ActionNumber2",
        3: "PF2E.ActionNumber3",
    },

    actionCategories: {
        interaction: "PF2E.ActionCategoryInteraction",
        defensive: "PF2E.ActionCategoryDefensive",
        offensive: "PF2E.ActionCategoryOffensive",
    },

    frequencies: {
        PT1M: "PF2E.Duration.PT1M",
        PT10M: "PF2E.Duration.PT10M",
        PT1H: "PF2E.Duration.PT1H",
        PT24H: "PF2E.Duration.PT24H",
        day: "PF2E.Duration.day",
        P1W: "PF2E.Duration.P1W",
    },

    // Proficiency Multipliers
    proficiencyLevels: [
        "PF2E.ProficiencyLevel0", // untrained
        "PF2E.ProficiencyLevel1", // trained
        "PF2E.ProficiencyLevel2", // expert
        "PF2E.ProficiencyLevel3", // master
        "PF2E.ProficiencyLevel4", // legendary
    ] as const,

    actorSizes: sizeTypes,

    actorTypes,

    speedTypes: {
        swim: "PF2E.SpeedTypesSwim",
        climb: "PF2E.SpeedTypesClimb",
        fly: "PF2E.SpeedTypesFly",
        burrow: "PF2E.SpeedTypesBurrow",
    },

    prerequisitePlaceholders: {
        prerequisite1: "PF2E.Prerequisite1",
        prerequisite2: "PF2E.Prerequisite2",
        prerequisite3: "PF2E.Prerequisite3",
        prerequisite4: "PF2E.Prerequisite4",
        prerequisite5: "PF2E.Prerequisite5",
    },

    senses,

    senseAcuity,

    bulkTypes: {
        L: "PF2E.BulkTypeLight",
        1: "1",
        2: "2",
        3: "3",
        4: "4",
        5: "5",
        6: "6",
        7: "7",
        8: "8",
        9: "9",
        10: "10",
        11: "11",
        12: "12",
        13: "13",
        14: "14",
        15: "15",
        16: "16",
        17: "17",
        18: "18",
        19: "19",
        20: "20",
        21: "21",
        22: "22",
        23: "23",
        24: "24",
        25: "25",
        26: "26",
        27: "27",
        28: "28",
        29: "29",
        30: "30",
        31: "31",
        32: "32",
        33: "33",
        34: "34",
        35: "35",
        36: "36",
        37: "37",
        38: "38",
        39: "39",
        40: "40",
        41: "41",
        42: "42",
        43: "43",
        44: "44",
        45: "45",
        46: "46",
        47: "47",
        48: "48",
        49: "49",
        50: "50",
    },

    conditionTypes,

    pfsFactions: {
        EA: "PF2E.PFS.Factions.EA",
        GA: "PF2E.PFS.Factions.GA",
        HH: "PF2E.PFS.Factions.HH",
        VS: "PF2E.PFS.Factions.VS",
        RO: "PF2E.PFS.Factions.RO",
        VW: "PF2E.PFS.Factions.VW",
    },

    pfsSchools: {
        none: "PF2E.PFS.School.None",
        scrolls: "PF2E.PFS.School.Scrolls",
        spells: "PF2E.PFS.School.Spells",
        swords: "PF2E.PFS.School.Swords",
    },

    immunityTypes,

    // Languages, alphabetical by common, uncommon, secret
    languages: {
        common: "PF2E.LanguageCommon",
        draconic: "PF2E.LanguageDraconic",
        dwarven: "PF2E.LanguageDwarven",
        elven: "PF2E.LanguageElven",
        gnomish: "PF2E.LanguageGnomish",
        goblin: "PF2E.LanguageGoblin",
        halfling: "PF2E.LanguageHalfling",
        jotun: "PF2E.LanguageJotun",
        orcish: "PF2E.LanguageOrcish",
        sylvan: "PF2E.LanguageSylvan",
        undercommon: "PF2E.LanguageUndercommon",
        ysoki: "PF2E.LanguageYsoki",
        abyssal: "PF2E.LanguageAbyssal",
        adlet: "PF2E.LanguageAdlet",
        aklo: "PF2E.LanguageAklo",
        akitonian: "PF2E.LanguageAkitonian",
        alghollthu: "PF2E.LanguageAlghollthu",
        amurrun: "PF2E.LanguageAmurrun",
        anadi: "PF2E.LanguageAnadi",
        anugobu: "PF2E.LanguageAnugobu",
        arcadian: "PF2E.LanguageArcadian",
        aquan: "PF2E.LanguageAquan",
        arboreal: "PF2E.LanguageArboreal",
        auran: "PF2E.LanguageAuran",
        boggard: "PF2E.LanguageBoggard",
        calda: "PF2E.LanguageCalda",
        caligni: "PF2E.LanguageCaligni",
        celestial: "PF2E.LanguageCelestial",
        cyclops: "PF2E.LanguageCyclops",
        daemonic: "PF2E.LanguageDaemonic",
        destrachan: "PF2E.LanguageDestrachan",
        dziriak: "PF2E.LanguageDziriak",
        ekujae: "PF2E.LanguageEkujae",
        erutaki: "PF2E.LanguageErutaki",
        formian: "PF2E.LanguageFormian",
        garundi: "PF2E.LanguageGarundi",
        girtablilu: "PF2E.LanguageGirtablilu",
        gnoll: "PF2E.LanguageGnoll",
        goloma: "PF2E.LanguageGoloma",
        grippli: "PF2E.LanguageGrippli",
        hallit: "PF2E.LanguageHallit",
        hwan: "PF2E.LanguageHwan",
        ignan: "PF2E.LanguageIgnan",
        ikeshti: "PF2E.LanguageIkeshti",
        infernal: "PF2E.LanguageInfernal",
        iruxi: "PF2E.LanguageIruxi",
        jistkan: "PF2E.LanguageJistkan",
        jyoti: "PF2E.LanguageJyoti",
        kaava: "PF2E.LanguageKaava",
        kibwani: "PF2E.LanguageKibwani",
        kitsune: "PF2E.LanguageKitsune",
        kelish: "PF2E.LanguageKelish",
        lirgeni: "PF2E.LanguageLirgeni",
        mahwek: "PF2E.LanguageMahwek",
        minaten: "PF2E.LanguageMinaten",
        minkaian: "PF2E.LanguageMinkaian",
        mwangi: "PF2E.LanguageMwangi",
        mzunu: "PF2E.LanguageMzunu",
        nagaji: "PF2E.LanguageNagaji",
        necril: "PF2E.LanguageNecril",
        ocotan: "PF2E.LanguageOcotan",
        okaiyan: "PF2E.LanguageOkaiyan",
        osiriani: "PF2E.LanguageOsiriani",
        protean: "PF2E.LanguageProtean",
        rasu: "PF2E.LanguageRasu",
        razatlani: "PF2E.LanguageRazatlani",
        requian: "PF2E.LanguageRequian",
        russian: "PF2E.LanguageRussian",
        senzar: "PF2E.LanguageSenzar",
        shadowtongue: "PF2E.LanguageShadowtongue",
        shobhad: "PF2E.LanguageShobhad",
        shisk: "PF2E.LanguageShisk",
        shoanti: "PF2E.LanguageShoanti",
        shoony: "PF2E.LanguageShoony",
        skald: "PF2E.LanguageSkald",
        sphinx: "PF2E.LanguageSphinx",
        strix: "PF2E.LanguageStrix",
        taldane: "PF2E.LanguageTaldane",
        tekritanin: "PF2E.LanguageTekritanin",
        tengu: "PF2E.LanguageTengu",
        terran: "PF2E.LanguageTerran",
        thassilonian: "PF2E.LanguageThassilonian",
        tien: "PF2E.LanguageTien",
        utopian: "PF2E.LanguageUtopian",
        vanara: "PF2E.LanguageVanara",
        varisian: "PF2E.LanguageVarisian",
        varki: "PF2E.LanguageVarki",
        vishkanyan: "PF2E.LanguageVishkanyan",
        vudrani: "PF2E.LanguageVudrani",
        wyrwood: "PF2E.LanguageWyrwood",
        xanmba: "PF2E.LanguageXanmba",
        androffan: "PF2E.LanguageAndroffan",
        azlanti: "PF2E.LanguageAzlanti",
        grioth: "PF2E.LanguageGrioth",
        kovintal: "PF2E.LanguageKovintal",
        migo: "PF2E.LanguageMiGo",
        munavri: "PF2E.LanguageMunavri",
        samsaran: "PF2E.LanguageSamsaran",
        sasquatch: "PF2E.LanguageSasquatch",
        shae: "PF2E.LanguageShae",
        yithian: "PF2E.LanguageYithian",
        druidic: "PF2E.LanguageDruidic",
    },

    attackEffects: {
        grab: "PF2E.AttackEffectGrab",
        "improved-grab": "PF2E.AttackEffectImprovedGrab",
        constrict: "PF2E.AttackEffectConstrict",
        "greater-constrict": "PF2E.AttackEffectGreaterConstrict",
        knockdown: "PF2E.AttackEffectKnockdown",
        "improved-knockdown": "PF2E.AttackEffectImprovedKnockdown",
        push: "PF2E.AttackEffectPush",
        "improved-push": "PF2E.AttackEffectImprovedPush",
        trip: "PF2E.AttackEffectTrip",
    },

    // Year offsets relative to the current actual year
    worldClock: {
        AR: { yearOffset: 2700 },
        IC: { yearOffset: 5200 },
        AD: { yearOffset: -95 },
        CE: { yearOffset: 0 },
    },

    runes: {
        weapon: {
            property: { ...WEAPON_PROPERTY_RUNES },
        },
    },

    monsterAbilities: () => {
        return {
            "All-Around Vision": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>This monster can see in all directions simultaneously, and therefore can\'t be flanked.',
            },
            "Aquatic Ambush": {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br><b>Requirements</b> The monster is hiding in water and a creature that hasn\'t detected it is within the listed number of feet. <b>Effect</b> The monster moves up to its swim Speed + 10 feet toward the triggering creature, traveling on water and on land. Once the creature is in reach, the monster makes a Strike against it. The creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=16">flat-footed</a> against this Strike.',
            },
            "Attack of Opportunity": {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br><b>Trigger</b> A creature within the monster\'s reach uses a manipulate action or a move action, makes a ranged attack, or leaves a square during a move action it\'s using. <b>Effect</b> The monster attempts a melee Strike against the triggering creature. If the attack is a critical hit and the trigger was a manipulate action, the monster disrupts that action. This Strike doesn\'t count toward the monster\'s multiple attack penalty, and its multiple attack penalty doesn\'t apply to this Strike.',
            },
            "At-Will Spells": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>The monster can cast its at-will spells any number of times without using up spell slots.',
            },
            Aura: {
                actionType: "passive",
                actionCost: 1,
                description:
                    "<b>Source</b> <a href=\"https://paizo.com/products/btq01y0m?Pathfinder-Bestiary\" target=\"_blank\" class=\"external-link\"><i>Bestiary pg. 342</i></a><br>A monster's aura automatically affects everything within a specified emanation around that monster. The monster doesn't need to spend actions on the aura; rather, the aura's effects are applied at specific times, such as when a creature ends its turn in the aura or when creatures enter the aura.<br><br> If an aura does nothing but deal damage, its entry lists only the radius, damage, and saving throw. Such auras deal this damage to a creature when the creature enters the aura and when a creature starts its turn in the aura. A creature can take damage from the aura only once per round.<br><br> The GM might determine that a monster's aura doesn't affect its own allies. For example, a creature might be immune to a monster's frightful presence if they have been around each other for a long time.",
            },
            Buck: {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>Most monsters that serve as mounts can attempt to\nbuck off unwanted or annoying riders, but most mounts\nwill not use this reaction against a trusted creature unless\nthe mounts are spooked or mistreated. <b>Trigger</b> A creature\nMounts or uses the Command an Animal action while riding\nthe monster. <b>Effect</b> The triggering creature must succeed\nat a Reflex saving throw against the listed DC or fall off the\ncreature and land prone. If the save is a critical failure, the\ntriggering creature also takes 1d6 bludgeoning damage in\naddition to the normal damage for the fall.',
            },
            "Catch Rock": {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br><b>Requirements</b> The monster must have a free hand but can Release anything it\'s holding as part of this reaction. <b>Trigger</b> The monster is targeted with a thrown rock Strike or a rock would fall on the monster. <b>Effect</b> The monster gains a +4 circumstance bonus to its AC against the triggering attack or to any defense against the falling rock. If the attack misses or the monster successfully defends against the falling rock, the monster catches the rock, takes no damage, and is now holding the rock.',
            },
            "Change Shape": {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>(<a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=32">concentrate</a>, [magical tradition], <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=127">polymorph</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=157">transmutation</a>) The monster changes its shape indefinitely. It can use this action again to return to its natural shape or adopt a new shape. Unless otherwise noted, a monster cannot use Change Shape to appear as a specific individual. Using Change Shape counts as creating a disguise for the Impersonate use of Deception. The monster\'s transformation automatically defeats Perception DCs to determine whether the creature is a member of the ancestry or creature type into which it transformed, and it gains a +4 status bonus to its Deception DC to prevent others from seeing through its disguise. Change Shape abilities specify what shapes the monster can adopt. The monster doesn\'t gain any special abilities of the new shape, only its physical form. For example, in each shape, it replaces its normal Speeds and Strikes, and might potentially change its senses or size. Any changes are listed in its stat block.',
            },
            "Constant Spells": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>A constant spell affects the monster without the monster needing to cast it, and its duration is unlimited. If a constant spell gets counteracted, the monster can reactivate it by spending the normal spellcasting actions the spell\u001crequires.',
            },
            Constrict: {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>The monster deals the listed amount of damage to any number of creatures grabbed or restrained by it. Each of those creatures can attempt a basic Fortitude save with the listed DC.',
            },
            Coven: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 342</i></a><br>(<a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=47">divination</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=106">mental</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=120">occult</a>) This monster can form a coven with two or more other creatures who also have the coven ability. This involves performing an 8-hour ceremony with all prospective coven members. After the coven is formed, each of its members gains elite adjustments (page 6), adjusting their levels accordingly. Coven members can sense other members\' locations and conditions by spending a single action, which has the concentrate trait, and can sense what another coven member is sensing as a two-action activity, which has the concentrate trait as well.<br><br> Covens also grant spells and rituals to their members, but these can be cast only in cooperation between three coven members who are all within 30 feet of one another. A coven member can contribute to a coven spell with a single-action spellcasting activity that has a single verbal component. If two coven members have contributed these actions within the last round, a third member can cast a coven spell on her turn by spending the normal spellcasting actions. A coven can cast its coven spells an unlimited number of times but can cast only one coven spell each round. All covens grant the 8th-level <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=17">baleful polymorph</a></i> spell and all the following spells, which the coven can cast at any level up to 5th: <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=15">augury</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=34">charm</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=39">clairaudience</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=40">clairvoyance</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=90">dream message</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=159">illusory disguise</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=161">illusory scene</a></i>, <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=239">prying eye</a></i>, and <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=329">talking corpse</a></i>. Individual creatures with the coven ability also grant additional spells to any coven they join. A coven can also cast the <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Rituals.aspx?ID=9">control weather</a></i> ritual, with a DC of 23 instead of the standard DC.<br><br> If a coven member leaving the coven or the death of a coven member brings the coven below three members, the remaining members keep their elite adjustments for 24 hours, but without enough members to contribute the necessary actions, they can\'t cast coven spells.',
            },
            Darkvision: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>A monster with darkvision can see perfectly well in areas of darkness and dim light, though such vision is in black and white only. Some forms of magical darkness, such as a 4th-level <i><a style="text-decoration:underline" href="https://2e.aonprd.com/Spells.aspx?ID=59">darkness</a></i> spell, block normal darkvision. A monster with greater darkvision, however, can see through even these forms of magical darkness.',
            },
            Disease: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>When a creature is exposed to a monster\'s disease, it attempts a Fortitude save or succumbs to the disease. The level of a disease is the level of the monster in\u001f icting the disease. The disease follows the <a style="text-decoration:underline" href="https://2e.aonprd.com/Rules.aspx?ID=361">rules for afflictions</a>.',
            },
            Engulf: {
                actionType: "action",
                actionCost: 2,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster Strides up to double its Speed and can move through the spaces of any creatures in its path. Any creature of the monster\'s size or smaller whose space the monster moves through can attempt a Reflex save with the listed DC to avoid being engulfed. A creature unable to act automatically critically fails this save. If a creature succeeds at its save, it can choose to be either pushed aside (out of the monster\'s path) or pushed in front of the monster to the end of the monster\'s movement. The monster can attempt to Engulf the same creature only once in a single use of Engulf. The monster can contain as many creatures as can fit in its space.<br><br> A creature that fails its save is pulled into the monster\'s body. It is grabbed, is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=35">slowed 1</a>, and has to hold its breath or start suffocating. The creature takes the listed amount of damage when first engulfed and at the end of each of its turns while it\'s engulfed. An engulfed creature can get free by Escaping against the listed escape DC. An engulfed creature can attack the monster engulfing it, but only with unarmed attacks or with weapons of light Bulk or less. The engulfing creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=16">flat-footed</a> against the attack. If the monster takes piercing or slashing damage equaling or exceeding the listed Rupture value from a single attack or spell, the engulfed creature cuts itself free. A creature that gets free by either method can immediately breathe and exits the swallowing monster\'s space.<br><br> If the monster dies, all creatures it has engulfed are automatically released as the monster\'s form loses cohesion.',
            },
            "Fast Healing": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>A monster with this ability regains the given number of Hit Points each round at the beginning of its turn.',
            },
            Ferocity: {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br><b>Trigger</b> The monster is reduced to 0 HP. <b>Effect</b> The monster avoids being knocked out and remains at 1 HP, but its <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=42">wounded</a> value increases by 1. When it is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=42">wounded 3</a>, it can no longer use this ability.',
            },
            "Frightful Presence": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>(<a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=206">aura</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=60">emotion</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=68">fear</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=106">mental</a>) A creature that first enters the area must attempt a Will save. Regardless of the result of the saving throw, the creature is temporarily immune to this monster\'s Frightful Presence for 1 minute.<br><br> <b>Critical Success</b> The creature is unaffected by the presence.<br> <b>Success</b> The creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=19">frightened 1</a>.<br> <b>Failure</b> The creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=19">frightened 2</a>.<br> <b>Critical Failure</b> The creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=19">frightened 4</a>.',
            },
            Grab: {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br><b>Requirements</b> The monster\'s last action was a success with a Strike that lists Grab in its damage entry, or it has a creature grabbed using this action. <b>Effect</b> The monster automatically Grabs the target until the end of the monster\'s next turn. The creature is grabbed by whichever body part the monster attacked with, and that body part can\'t be used to Strike creatures until the grab is ended.<br><br> Using Grab extends the duration of the monster\'s Grab until the end of its next turn for all creatures grabbed by it. A grabbed creature can use the <a style="text-decoration:underline" href="https://2e.aonprd.com/Actions.aspx?ID=79">Escape</a> action to get out of the grab, and the Grab ends for a grabbed creatures if the monster moves away from it.',
            },
            "Greater Constrict": {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster deals the listed amount of damage to any number of creatures grabbed or restrained by it. Each of those creatures can attempt a basic Fortitude save with the listed DC. A creature that fails this save falls <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=38">unconscious</a>, and a creature that succeeds is then temporarily immune to falling <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=38">unconscious</a> from Greater Constrict for 1 minute.',
            },
            "Improved Grab": {
                actionType: "free",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster can use Grab as a free action triggered by a hit with its initial attack. A monster with Improved Grab still needs to spend an action to extend the duration for creatures it already has grabbed.',
            },
            "Improved Knockdown": {
                actionType: "free",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster can use Knockdown as a free action triggered by a hit with its initial attack.',
            },
            "Improved Push": {
                actionType: "free",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster can use Push as a free action triggered by a hit with its initial attack.',
            },
            Knockdown: {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br><b>Requirements</b> The monster\'s last action was a success with a Strike that lists Knockdown in its damage entry. <b>Effect</b> The monster knocks the target prone.',
            },
            Lifesense: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>Lifesense allows a monster to sense the vital essence of living and undead creatures within the listed range. The sense can distinguish between the positive energy animating living creatures and the negative energy animating undead creatures, much as sight distinguishes colors.',
            },
            "Light Blindness": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>When first exposed to bright light, the monster is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=1">blinded</a> until the end of its next turn. After this exposure, light doesn\'t blind the monster again until after it spends 1 hour in darkness. However, as long as the monster is in an area of bright light, it\'s <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=7">dazzled</a>.',
            },
            "Low-Light Vision": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>The monster can see in dim light as though it were bright light, so it ignores the <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=4">concealed</a> condition due to dim light.',
            },
            Poison: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>When a creature is exposed to a monster\'s poison, it attempts a Fortitude save to avoid becoming poisoned. The level of a poison is the level of the monster inflicting the poison. The poison follows the <a style="text-decoration:underline" href="https://2e.aonprd.com/Rules.aspx?ID=361">rules for afflictions</a>.',
            },
            Push: {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br><b>Requirements</b> The monster\'s last action was a success with a Strike that lists Push in its damage entry. <b>Effect</b> The monster automatically knocks the target away from the monster. Unless otherwise noted in the ability description, the creature is pushed 5 feet. If the attack was a critical hit, this distance is doubled.',
            },
            Regeneration: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 343</i></a><br>This monster regains the listed number of Hit Points each round at the beginning of its turn. Its <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=11">dying</a> condition never increases beyond <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=11">dying 3</a> as long as its regeneration is active. However, if it takes damage of a type listed in the regeneration entry, its regeneration deactivates until the end of its next turn. Deactivate the regeneration before applying any damage of a listed type, since that damage might kill the monster by bringing it to <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=11">dying 4</a>.',
            },
            Rend: {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>A Rend entry lists a Strike the monster has. <b>Requirements</b> The monster hit the same enemy with two consecutive Strikes of the listed type in the same round. <b>Effect</b> The monster automatically deals that Strike\'s damage again to the enemy.',
            },
            "Retributive Strike": {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br><b>Trigger</b> An enemy damages the monster\'s ally, and both are within 15 feet of the monster. <b>Effect</b> The ally gains resistance to all damage against the triggering damage equal to 2 + the monster\'s level. If the foe is within reach, the monster makes a melee Strike against it.',
            },
            Scent: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>Scent involves sensing creatures or objects by smell, and is usually a vague sense. The range is listed in the ability, and it functions only if the creature or object being detected emits an aroma (for instance, incorporeal creatures usually do not exude an aroma).<br><br> If a creature emits a heavy aroma or is upwind, the GM can double or even triple the range of scent abilities used to detect that creature, and the GM can reduce the range if a creature is downwind.',
            },
            "Shield Block": {
                actionType: "reaction",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br><b>Trigger</b> The monster has its shield raised and takes damage from a physical attack. <b>Effect</b> The monster snaps its shield into place to deflect a blow. The shield prevents the monster from taking an amount of damage up to the shield\'s Hardness. The monster and the shield each take any remaining damage, possibly breaking or destroying the shield.',
            },
            "Sneak Attack": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>When the monster Strikes a creature that has the <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=16">flat-footed</a> condition with an <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=170">agile</a> or <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=179">finesse</a> melee weapon, an <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=170">agile</a> or <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=179">finesse</a> unarmed attack, or a ranged weapon attack, it also deals the listed precision damage. For a ranged attack with a thrown weapon, that weapon must also be an <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=170">agile</a> or <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=179">finesse</a> weapon.',
            },
            "Swallow Whole": {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>(<a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=15">attack</a>) The monster attempts to swallow a creature of the listed size or smaller that it has grabbed in its jaws or mouth. If a swallowed creature is of the maximum size listed, the monster can\'t use Swallow Whole again. If the creature is smaller than the maximum, the monster can usually swallow more creatures; the GM determines the maximum. The monster attempts an <a style="text-decoration:underline" href="https://2e.aonprd.com/Skills.aspx?ID=3">Athletics</a> check opposed by the grabbed creature\'s Reflex DC. If it succeeds, it swallows the creature. The monster\'s mouth or jaws no longer grab a creature it has swallowed, so the monster is free to use them to Strike or Grab once again. The monster can\'t attack creatures it has swallowed.<br><br> A swallowed creature is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=20">grabbed</a>, is <a style="text-decoration:underline" href="https://2e.aonprd.com/Conditions.aspx?ID=35">slowed 1</a>, and has to hold its breath or start suffocating. The swallowed creature takes the listed amount of damage when first swallowed and at the end of each of its turns while it\'s swallowed. If the victim Escapes this ability\'s grabbed condition, it exits through the monster\'s mouth. This frees any other creature grabbed in the monster\'s mouth or jaws. A swallowed creature can attack the monster that has swallowed it, but only with unarmed attacks or with weapons of light Bulk or less. The engulfing creature is flat-footed against the attack. If the monster takes piercing or slashing damage equaling or exceeding the listed Rupture value from a single attack or spell, the engulfed creature cuts itself free. A creature that gets free by either Escaping or cutting itself free can immediately breathe and exits the swallowing monster\'s space.<br><br> If the monster dies, a swallowed creature can be freed by creatures adjacent to the corpse if they spend a combined total of 3 actions cutting the monster open with a weapon or unarmed attack that deals piercing or slashing damage.',
            },
            "Swarm Mind": {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>This monster doesn\'t have a single mind (typically because it\'s a swarm of smaller creatures), and is immune to mental effects that target only a specific number of creatures. It is still subject to mental effects that affect all creatures in an area.',
            },
            Telepathy: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>(<a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=206">aura</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=47">divination</a>, <a style="text-decoration:underline" href="https://2e.aonprd.com/Traits.aspx?ID=103">magical</a>) A monster with telepathy can communicate mentally with any creatures within the listed radius, as long as they share a language. This doesn\'t give any special access to their thoughts, and communicates no more information than normal speech would.',
            },
            "Throw Rock": {
                actionType: "action",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>The monster picks up a rock within reach or retrieves a stowed rock and throws it, making a ranged Strike.',
            },
            Trample: {
                actionType: "action",
                actionCost: 3,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>The monster Strides up to double its Speed and can move through the spaces of creatures of the listed size, Trampling each creature whose space it enters. The monster can attempt to Trample the same creature only once in a single use of Trample. The monster deals the damage of the listed Strike, but trampled creatures can attempt a basic Reflex save at the listed DC (no damage on a critical success, half damage on a success, double damage on a critical failure).',
            },
            Tremorsense: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>Tremorsense allows a monster to feel the vibrations through a solid surface caused by movement. It is an imprecise sense with a limited range (listed in the ability). Tremorsense functions only if the monster is on the same surface as the subject, and only if the subject is moving along (or burrowing through) the surface.',
            },
            Wavesense: {
                actionType: "passive",
                actionCost: 1,
                description:
                    '<b>Source</b> <a href="https://paizo.com/products/btq01y0m?Pathfinder-Bestiary" target="_blank" class="external-link"><i>Bestiary pg. 344</i></a><br>This sense allows a monster to feel vibrations caused by movement through a liquid. It\'s an imprecise sense with a limited range (listed in the ability). Wavesense functions only if monster and the subject are in the same body of liquid, and only if the subject is moving through the liquid.',
            },
        };
    },

    SETTINGS: {
        automation: {
            rulesBasedVision: {
                name: "PF2E.SETTINGS.Automation.RulesBasedVision.Name",
                hint: "PF2E.SETTINGS.Automation.RulesBasedVision.Hint",
            },
            effectExpiration: {
                name: "PF2E.SETTINGS.Automation.EffectExpiration.Name",
                hint: "PF2E.SETTINGS.Automation.EffectExpiration.Hint",
            },
            removeExpiredEffects: {
                name: "PF2E.SETTINGS.Automation.RemoveExpiredEffects.Name",
                hint: "PF2E.SETTINGS.Automation.RemoveExpiredEffects.Hint",
            },
            flankingDetection: {
                name: "PF2E.SETTINGS.Automation.FlankingDetection.Name",
                hint: "PF2E.SETTINGS.Automation.FlankingDetection.Hint",
            },
            actorsDeadAtZero: {
                name: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.Name",
                hint: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.Hint",
                neither: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.Neither",
                npcsOnly: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.NPCsOnly",
                pcsOnly: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.PCsOnly",
                both: "PF2E.SETTINGS.Automation.ActorsDeadAtZero.Both",
            },
            lootableNPCs: {
                name: "PF2E.SETTINGS.Automation.LootableNPCs.Name",
                hint: "PF2E.SETTINGS.Automation.LootableNPCs.Hint",
            },
        },
        homebrew: {
            creatureTraits: {
                name: "PF2E.SETTINGS.Homebrew.CreatureTraits.Name",
                hint: "PF2E.SETTINGS.Homebrew.CreatureTraits.Hint",
            },
            featTraits: {
                name: "PF2E.SETTINGS.Homebrew.FeatTraits.Name",
                hint: "PF2E.SETTINGS.Homebrew.FeatTraits.Hint",
            },
            languages: {
                name: "PF2E.SETTINGS.Homebrew.Languages.Name",
                hint: "PF2E.SETTINGS.Homebrew.Languages.Hint",
            },
            magicSchools: {
                name: "PF2E.SETTINGS.Homebrew.MagicSchools.Name",
                hint: "PF2E.SETTINGS.Homebrew.MagicSchools.Hint",
            },
            spellTraits: {
                name: "PF2E.SETTINGS.Homebrew.SpellTraits.Name",
                hint: "PF2E.SETTINGS.Homebrew.SpellTraits.Hint",
            },
            weaponCategories: {
                name: "PF2E.SETTINGS.Homebrew.WeaponCategories.Name",
                hint: "PF2E.SETTINGS.Homebrew.WeaponCategories.Hint",
            },
            weaponGroups: {
                name: "PF2E.SETTINGS.Homebrew.WeaponGroups.Name",
                hint: "PF2E.SETTINGS.Homebrew.WeaponGroups.Hint",
            },
            baseWeapons: {
                name: "PF2E.SETTINGS.Homebrew.BaseWeapons.Name",
                hint: "PF2E.SETTINGS.Homebrew.BaseWeapons.Hint",
            },
            weaponTraits: {
                name: "PF2E.SETTINGS.Homebrew.WeaponTraits.Name",
                hint: "PF2E.SETTINGS.Homebrew.WeaponTraits.Hint",
            },
            equipmentTraits: {
                name: "PF2E.SETTINGS.Homebrew.EquipmentTraits.Name",
                hint: "PF2E.SETTINGS.Homebrew.EquipmentTraits.Hint",
            },
        },
        worldClock: {
            name: "PF2E.SETTINGS.WorldClock.Name",
            label: "PF2E.SETTINGS.WorldClock.Label",
            hint: "PF2E.SETTINGS.WorldClock.Hint",
            dateTheme: {
                name: "PF2E.SETTINGS.WorldClock.DateTheme.Name",
                hint: "PF2E.SETTINGS.WorldClock.DateTheme.Hint",
                AR: "PF2E.SETTINGS.WorldClock.DateTheme.AR",
                AD: "PF2E.SETTINGS.WorldClock.DateTheme.AD",
                CE: "PF2E.SETTINGS.WorldClock.DateTheme.CE",
            },
            timeConvention: {
                name: "PF2E.SETTINGS.WorldClock.TimeConvention.Name",
                hint: "PF2E.SETTINGS.WorldClock.TimeConvention.Hint",
                twentyFour: "PF2E.SETTINGS.WorldClock.TimeConvention.TwentyFour",
                twelve: "PF2E.SETTINGS.WorldClock.TimeConvention.Twelve",
            },
            showClockButton: {
                name: "PF2E.SETTINGS.WorldClock.ShowClockButton.Name",
                hint: "PF2E.SETTINGS.WorldClock.ShowClockButton.Hint",
            },
            playersCanView: {
                name: "PF2E.SETTINGS.WorldClock.PlayersCanView.Name",
                hint: "PF2E.SETTINGS.WorldClock.PlayersCanView.Hint",
            },
            syncDarkness: {
                name: "PF2E.SETTINGS.WorldClock.SyncDarkness.Name",
                hint: "PF2E.SETTINGS.WorldClock.SyncDarkness.Hint",
            },
            syncDarknessScene: {
                name: "PF2E.SETTINGS.WorldClock.SyncDarknessScene.Name",
                hint: "PF2E.SETTINGS.WorldClock.SyncDarknessScene.Hint",
                enabled: "PF2E.SETTINGS.WorldClock.SyncDarknessScene.Enabled",
                disabled: "PF2E.SETTINGS.WorldClock.SyncDarknessScene.Disabled",
                default: "PF2E.SETTINGS.WorldClock.SyncDarknessScene.Default",
            },
            worldCreatedOn: {
                name: "PF2E.SETTINGS.WorldClock.WorldCreatedOn.Name",
                hint: "PF2E.SETTINGS.WorldClock.WorldCreatedOn.Hint",
            },
        },
        CampaignFeats: {
            name: "PF2E.SETTINGS.CampaignFeats.Name",
            hint: "PF2E.SETTINGS.CampaignFeats.Hint",
        },
    },

    Actor: {
        documentClasses: {
            character: CharacterPF2e,
            npc: NPCPF2e,
            hazard: HazardPF2e,
            loot: LootPF2e,
            familiar: FamiliarPF2e,
            vehicle: VehiclePF2e,
        },
    },

    Item: {
        documentClasses: {
            action: ActionItemPF2e,
            ancestry: AncestryPF2e,
            armor: ArmorPF2e,
            background: BackgroundPF2e,
            backpack: ContainerPF2e,
            book: BookPF2e,
            class: ClassPF2e,
            condition: ConditionPF2e,
            consumable: ConsumablePF2e,
            deity: DeityPF2e,
            effect: EffectPF2e,
            equipment: EquipmentPF2e,
            feat: FeatPF2e,
            heritage: HeritagePF2e,
            kit: KitPF2e,
            lore: LorePF2e,
            melee: MeleePF2e,
            spell: SpellPF2e,
            spellcastingEntry: SpellcastingEntryPF2e,
            treasure: TreasurePF2e,
            weapon: WeaponPF2e,
        },
        traits: {
            action: actionTraits,
            armor: armorTraits,
            ancestry: creatureTraits,
            backpack: equipmentTraits,
            book: equipmentTraits,
            consumable: consumableTraits,
            equipment: equipmentTraits,
            feat: featTraits,
            heritage: featTraits,
            kit: classTraits,
            melee: npcAttackTraits,
            spell: spellTraits,
            weapon: weaponTraits,
        },
    },

    JournalEntry: { sheetClass: JournalSheetPF2e },
};
